
/*=============================================================================*\
 * File: gfsmAlgebra.h
 * Author: Bryan Jurish <moocow@ling.uni-potsdam.de>
 * Description: finite state machine library
 *
 * Copyright (c) 2004 Bryan Jurish.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * See file LICENSE for further informations on licensing terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *=============================================================================*/

#ifndef _GFSM_ALGEBRA_H
#define _GFSM_ALGEBRA_H

#include <glib.h>
#include <gfsmAutomaton.h>
#include <gfsmAlphabet.h>

/*======================================================================
 * Methods: algebra
 */
///\name Algebra
//@{

//------------------------------
/** Compute transitive (@is_plus!=FALSE) or reflexive+transitive (@is_plus==FALSE)
 *  closure of @fsm.  Destructively alters @fsm1.
 *  \returns @fsm
 */
gfsmAutomaton *gfsm_automaton_closure(gfsmAutomaton *fsm, gboolean is_plus);

/** Final-state pre-traversal utility for closure(fsm). */
gboolean _gfsm_automaton_closure_final_func(gfsmStateId id, gpointer dummy, gfsmAutomaton *fsm);

//------------------------------
/**
 * Compute the complement of @fsm with respect to its own alphabet.
 * Destructively alters @fsm \returns @fsm
 */
gfsmAutomaton *gfsm_automaton_complement(gfsmAutomaton *fsm);

/**
 * Compute the complement of @fsm with respect to the alphabet @alph,
 * which should contain all of the labels from @fsm1.  Destructively alters @fsm
 * WIP
 * \returns @fsm
 */
gfsmAutomaton *gfsm_automaton_complement_full(gfsmAutomaton *fsm, gfsmAlphabet *alph);

/**
 * Complete the automaton @fsm with respect to the alphabet @alph
 * by directing missing arcs to the state with id @sink.  Destructively
 * alters @fsm.
 * WIP
 * \returns @fsm
 */
gfsmAutomaton *gfsm_automaton_complete(gfsmAutomaton *fsm, gfsmAlphabet *alph, gfsmStateId sink);

//------------------------------
/** Append @fsm2 onto the end of @fsm1.  \returns @fsm1 */
gfsmAutomaton *gfsm_automaton_concat(gfsmAutomaton *fsm1, const gfsmAutomaton *fsm2);

/** Final-state pre-traversal utility for concat(fsm,fsm2).
 *  Assumes (fsm->root_id) has been temporarily set to the translated stateId
 *  of fsm2->root_id */
gboolean _gfsm_automaton_concat_final_func(gfsmStateId id, gpointer dummy, gfsmAutomaton *fsm);

//------------------------------
/** Invert upper and lower labels of an FSM */
gfsmAutomaton *gfsm_automaton_invert(gfsmAutomaton *fsm);

//------------------------------
/** Project lower (@which == 1) or upper (@which != 1) tape of @fsm */
gfsmAutomaton *gfsm_automaton_project(gfsmAutomaton *fsm, int which);

//------------------------------
/** Reverse an @fsm. \returns @fsm */
gfsmAutomaton *gfsm_automaton_reverse(gfsmAutomaton *fsm);

//------------------------------
/** Assign the union of @fsm1 and @fsm2 to @fsm1. \returns @fsm1 */
gfsmAutomaton *gfsm_automaton_union(gfsmAutomaton *fsm1, const gfsmAutomaton *fsm2);
//@}

/** \todo
 *  Algebra:
 *  + complement(w.r.t. sigma2 >= sigma1)
 *    -> find_arc()
 *  + difference: A-B      :=   A \cap (!B) = !(!a \cup B)
 *  + intersect : A \cap B := !(!A \cup !B)
 *  + product
 *  + sigma
 *  + compose
 *  + bestpath
 *  + paths
 *  + ? encode
 *  + ? equiv
 *  + ? determinize
 *  + ? minimize
 *  + ? prune
 *  + ? rmepsilon
 */

#endif /* _GFSM_ALGEBRA_H */
