
/*=============================================================================*\
 * File: gfsmArc.c
 * Author: Bryan Jurish <moocow@ling.uni-potsdam.de>
 * Description: finite state machine library: arcs
 *
 * Copyright (c) 2004 Bryan Jurish.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * See file LICENSE for further informations on licensing terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *=============================================================================*/

#include <gfsmArc.h>
#include <stdlib.h>

/*======================================================================
 * Methods: Arcs: Constructors etc.
 */

/*--------------------------------------------------------------
 * arc_init()
 */
gfsmArc *gfsm_arc_init(gfsmArc *a,
		       gfsmStateId dst,
		       gfsmLabelId lo,
		       gfsmLabelId hi,
		       gfsmWeight wt)
{
  a->target = dst;
  a->lower  = lo;
  a->upper  = hi;
  a->weight = wt;
  return a;
}


/*======================================================================
 * Methods: Arc Lists: Constructors etc.
 */

/*--------------------------------------------------------------
 * arclist_copy()
 */
gfsmArcList *gfsm_arclist_copy(const gfsmArcList *src)
{
  gfsmArcList *first=NULL, *cur, *last=NULL;
  for (; src != NULL; src=src->next, last=cur) {
    cur = gfsm_arclist_new_full(src->arc.target,
				src->arc.lower,
				src->arc.upper,
				src->arc.weight);
    cur->next = NULL;
    if (!first) first = cur;
    else last->next = cur;
  }
  return first;
}

/*--------------------------------------------------------------
 * arclist_free()
 */
void gfsm_arclist_free(gfsmArcList *al)
{
  gfsmArcList *next = al;
  for ( ; al != NULL; al = next) {
    next = al->next;
    g_free(al);
  }
}

/*======================================================================
 * Methods: Arc Lists: Utilities
 */
/*--------------------------------------------------------------
 * arclist_length()
 */
guint gfsm_arclist_length(gfsmArcList *al)
{
  guint n;
  for (n=0; al != NULL; al=al->next) ++n;
  return n;
}

/*--------------------------------------------------------------
 * compare()
 */
gint gfsm_arc_compare(gfsmArc *a1, gfsmArc *a2)
{
  if (!a1) {
    if (!a2) return 0;
    return 1;
  }
  if (!a2) return -1;

  if (a1->lower < a2->lower) return -1;
  else if (a1->lower > a2->lower) return 1;
  else if (a1->upper < a2->upper) return -1;
  else if (a1->upper > a2->upper) return 1;
  else if (a1->target < a2->target) return -1;
  else if (a1->target > a2->target) return 1;
  else if (a1->weight < a2->weight) return -1;
  else if (a1->weight > a2->weight) return 1;

  return 0;
}
