/* -*- Mode: C -*- */
/*=============================================================================*\
 * File: gfsmCommon.h
 * Author: Bryan Jurish <moocow@ling.uni-potsdam.de>
 * Description: finite state machine library: common definitions
 *
 * Copyright (c) 2004 Bryan Jurish.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * See file LICENSE for further informations on licensing terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *=============================================================================*/

#ifndef _GFSM_COMMON_H
#define _GFSM_COMMON_H

#include <glib.h>

/*----------------------------------------------------------------------
 * Types
 */
/** Type for elementary arc-labels */
typedef guint16 gfsmLabelId;

/** Type for extended arc-labels (parameters and return values) */
typedef guint32 gfsmLabelVal;

/** Type for elementary state identifiers */
typedef guint32 gfsmStateId;

/** Alias for state identifiers */
typedef gfsmStateId gfsmNodeId;

/** Type for arc weights */
typedef gfloat gfsmWeight;

/*----------------------------------------------------------------------
 * Constants
 */
/** Constant epsilon label */
extern const gfsmLabelId gfsmEpsilon;

/** Constant indicating missing alphabet key */
extern const gpointer    gfsmNoKey;

/** Constant indicating missing label */
extern const gfsmLabelId gfsmNoLabel;

/** Constant indicating missing state */
extern const gfsmStateId gfsmNoState;

/** Constant indicating missing weight */
extern const gfsmWeight gfsmNoWeight;

#endif /* _GFSM_COMMON_H */
